
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <time.h>
#include "garbage_collector.h"

struct vector {
	int *row;
	int len;
};

void run_test(unsigned num_tests);
char *rand_str(unsigned max_len);
char *concat(const char *str_1, const char *str_2);
struct vector rand_vector(unsigned len);
struct vector add_vector(const struct vector v_1, const struct vector v_2);
void print_vector(struct vector v);

DECLARE_GC(gc);
enum { STR_LEN_MAX = 20, VECTOR_LEN = 8, NUM_TESTS = 1000 };



int main(int argc, char *argv[])
{
	INITIALIZE_GC(gc);
	srand(time(NULL));

	run_test(NUM_TESTS);

	GC_FREE(gc);
  
  system("PAUSE");	
  return 0;
}




void run_test(unsigned num_tests) {
	char *str[2];
	struct vector v[2];
	int i;

	for (i = 0; i < num_tests; i++) {
		printf("TEST %d\n\n", i + 1);
		str[0] = rand_str(STR_LEN_MAX);
		str[1] = rand_str(STR_LEN_MAX);
		printf("%s + %s = %s\n\n", str[0], str[1], concat(str[0], str[1]));
		v[0] = rand_vector(VECTOR_LEN);
		v[1] = rand_vector(VECTOR_LEN);
		printf("  ");
		print_vector(v[0]);
		printf("\n+ ");
		print_vector(v[1]);
		printf("\n= ");
		print_vector(add_vector(v[0], v[1]));
		puts("\n\n");
	}
}

char *rand_str(unsigned max_len) {
	char *str;
	int len;
	int i;

	len = rand() % max_len + 1;
	GC_ALLOC(char, str, len + 1, gc);
	for (i = 0; i < len; i++) {
		str[i] = rand() % 26 + 'a';
	}
	str[i] = '\0';
	return str;
}

char *concat(const char *str_1, const char *str_2) {
	char *str;
	int str_1_len, len;
	int i;

	str_1_len = strlen(str_1);
	len = str_1_len + strlen(str_2);
	GC_ALLOC(char, str, len + 1, gc);
	for (i = 0; i < str_1_len; i++) {
		str[i] = str_1[i];
	}
	for (; i < len; i++) {
		str[i] = str_2[i - str_1_len];
	}
	str[i] = '\0';
	return str;
}

struct vector rand_vector(unsigned len) {
	struct vector v;
	int i;

	GC_ALLOC(int, v.row, len, gc);
	for (i = 0; i < len; i++) {
		v.row[i] = rand() % SHRT_MAX;
	}
	v.len = len;
	return v;
}

struct vector add_vector(const struct vector v_1, const struct vector v_2) {
	struct vector v;
	int i;

	if (v_1.len != v_2.len) {
		puts("different vector length");
		GC_FREE(gc);
		exit(EXIT_SUCCESS);
	}
	GC_ALLOC(int, v.row, v_1.len, gc);
	for (i = 0; i < v_1.len; i++) {
		v.row[i] = v_1.row[i] + v_2.row[i];
	}
	v.len = v_1.len;
	return v;
}

void print_vector(struct vector v) {
	int i;

	putchar('[');
	for (i = 0; i < v.len; i++) {
		printf("%d%s", v.row[i], i == v.len - 1 ? "" : ", ");
	}
	putchar(']');
}
